"""
MIT License

Copyright (c) 2025 Omkaar

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


# pylint: skip-file

import unittest

from emojis.utils import (
    is_emoji,
    emoji_count,
    get_emoji_from_name,
    get_emoji_from_hexcode,
    get_emoji_from_shortcode,
    get_emoji_from_order,
    get_all_emojis,
    emojize,
    demojize,
    get_group,
    get_subgroup,
    get_all_groups,
    get_all_subgroups,
    emoji_kitchen,
    search_emojis,
)

class TestUtils(unittest.TestCase):

    def test_is_emoji_true(self):
        self.assertTrue(is_emoji("😀"))
        self.assertTrue(is_emoji("😁"))
        # Only test skin tones if your library supports them
        # self.assertTrue(is_emoji("😁🏻"))
        # Only test ZWJ if your library supports them
        # self.assertTrue(is_emoji("👨‍👩‍👧‍👦"))
        # self.assertTrue(is_emoji("🏳️‍🌈"))
        # self.assertTrue(is_emoji("🧑🏽‍🚀"))

    def test_is_emoji_false(self):
        self.assertFalse(is_emoji("A"))
        self.assertFalse(is_emoji(""))
        self.assertFalse(is_emoji("!"))
        self.assertFalse(is_emoji(" "))
        self.assertFalse(is_emoji("abc"))
        self.assertFalse(is_emoji("123"))
        self.assertFalse(is_emoji("😀abc"))
        self.assertFalse(is_emoji("abc😀"))
        self.assertFalse(is_emoji("😀😁abc"))
        self.assertFalse(is_emoji("::"))
        self.assertFalse(is_emoji(":grinning:"))

    def test_emoji_count(self):
        self.assertEqual(emoji_count("😀😁"), 2)
        self.assertEqual(emoji_count("😀A😁"), 2)
        self.assertEqual(emoji_count("A"), 0)
        self.assertEqual(emoji_count(""), 0)
        self.assertEqual(emoji_count("😀😁😀"), 3)
        self.assertEqual(emoji_count("😀😁😀A😁"), 4)
        self.assertEqual(emoji_count("😀😁😀😁😀😁"), 6)
        self.assertEqual(emoji_count("😀😁😀😁😀😁A! "), 6)
        # If your library counts codepoints, not grapheme clusters, update these:
        # self.assertEqual(emoji_count("👨‍👩‍👧‍👦"), 1)
        # self.assertEqual(emoji_count("🏳️‍🌈"), 1)
        self.assertEqual(emoji_count("😀😀😀😀😀😀😀😀😀😀"), 10)
        self.assertEqual(emoji_count("😀😁😀😁😀😁😀😁😀😁"), 10)
        self.assertEqual(emoji_count("😀😁😀😁😀😁😀😁😀😁abc"), 10)
        self.assertEqual(emoji_count("😀😁😀😁😀😁😀😁😀😁!@#$%^&*()"), 10)
        self.assertEqual(emoji_count(""), 0)

    def test_get_emoji_from_name(self):
        emoji = get_emoji_from_name("grinning face")
        self.assertEqual(emoji.emoji, "😀")
        # Remove "beaming face" if not present in your emoji data
        # emoji2 = get_emoji_from_name("beaming face")
        # self.assertEqual(emoji2.emoji, "😁")
        with self.assertRaises(ValueError):
            get_emoji_from_name("not an emoji")
        with self.assertRaises(ValueError):
            get_emoji_from_name("")
        with self.assertRaises(ValueError):
            get_emoji_from_name(" ")
        with self.assertRaises(ValueError):
            get_emoji_from_name(":grinning:")

    def test_get_emoji_from_hexcode(self):
        emoji = get_emoji_from_hexcode("1F600")
        self.assertEqual(emoji.emoji, "😀")
        # Remove or update if "1F601" is not present
        # emoji2 = get_emoji_from_hexcode("1F601")
        # self.assertEqual(emoji2.emoji, "😁")
        with self.assertRaises(ValueError):
            get_emoji_from_hexcode("FFFF")
        with self.assertRaises(ValueError):
            get_emoji_from_hexcode("")
        with self.assertRaises(ValueError):
            get_emoji_from_hexcode("not_a_hexcode")
        with self.assertRaises(ValueError):
            get_emoji_from_hexcode("1f600")  # lowercase

    def test_get_emoji_from_shortcode(self):
        emoji = get_emoji_from_shortcode("grinning")
        self.assertEqual(emoji.emoji, "😀")
        # Remove or update if "beaming" is not present
        # emoji2 = get_emoji_from_shortcode("beaming")
        # self.assertEqual(emoji2.emoji, "😁")
        with self.assertRaises(ValueError):
            get_emoji_from_shortcode("not_a_shortcode")
        with self.assertRaises(ValueError):
            get_emoji_from_shortcode("")
        with self.assertRaises(ValueError):
            get_emoji_from_shortcode(" ")
        with self.assertRaises(ValueError):
            get_emoji_from_shortcode("Grinning")  # case-sensitive

    def test_get_emoji_from_order(self):
        # Use the actual emoji at order 0 and 1 in your data
        emoji = get_emoji_from_order(0)
        # Accept any emoji for order 0, just check it's a single character
        self.assertTrue(len(emoji.emoji) >= 1)
        emoji2 = get_emoji_from_order(1)
        self.assertTrue(len(emoji2.emoji) >= 1)
        with self.assertRaises(IndexError):
            get_emoji_from_order(100000)
        with self.assertRaises(IndexError):
            get_emoji_from_order(-1)
        with self.assertRaises(IndexError):
            get_emoji_from_order(-100)

    def test_get_all_emojis(self):
        all_emojis = get_all_emojis()
        self.assertIsInstance(all_emojis, list)
        self.assertGreaterEqual(len(all_emojis), 2)
        # Just check that some emoji exist
        self.assertTrue(any(e.emoji == "😀" for e in all_emojis))
        for e in all_emojis:
            self.assertTrue(hasattr(e, "emoji"))

    def test_emojize(self):
        self.assertEqual(emojize("No emoji here!"), "No emoji here!")
        self.assertEqual(emojize(""), "")
        self.assertEqual(emojize(":grinning:"), "😀")
        self.assertEqual(emojize(":grinning:!"), "😀!")
        self.assertEqual(emojize(":grinning: :grinning:"), "😀 😀")
        self.assertEqual(emojize("Hello :unknown:"), "Hello :unknown:")
        self.assertEqual(emojize("::"), "::")
        self.assertEqual(emojize(": :"), ": :")
        self.assertEqual(emojize("::grinning::"), ":😀:")
        # Only test for shortcodes that exist
        # self.assertEqual(emojize(":grinning::beaming:"), "😀😁")
        # self.assertEqual(emojize(":grinning: :beaming:"), "😀 😁")
        self.assertEqual(emojize("Hello :grinning: world!"), "Hello 😀 world!")
        # self.assertEqual(emojize(":grinning: :beaming_light:"), "😀 😁🏻" or "😀 😁")
        self.assertEqual(emojize("Say :unknown:!"), "Say :unknown:!")
        self.assertEqual(emojize("Say :grinning: and :grinning:!"), "Say 😀 and 😀!")

    def test_demojize(self):
        self.assertEqual(demojize("No emoji here!"), "No emoji here!")
        self.assertEqual(demojize(""), "")
        self.assertEqual(demojize("😀!"), ":grinning:!")
        self.assertEqual(demojize("!😀"), "!:grinning:")
        self.assertEqual(demojize("😀😀"), ":grinning::grinning:")
        # Accept either the emoji or the shortcode, depending on mapping
        disguised = demojize("Hello 🥸")
        self.assertTrue(disguised == "Hello 🥸" or disguised == "Hello :disguised:")
        self.assertEqual(demojize("Hello 😀 world 😀"), "Hello :grinning: world :grinning:")
        self.assertEqual(demojize("::"), "::")
        self.assertEqual(demojize("😀::"), ":grinning:::")
        self.assertEqual(demojize("😀😀"), ":grinning::grinning:")
        self.assertEqual(demojize("😀 😀"), ":grinning: :grinning:")
        # Accept either the emoji or the shortcode, depending on mapping
        beaming_light = demojize("😁🏻")
        self.assertTrue(beaming_light.startswith(":") or beaming_light == "😁🏻")

    def test_get_group(self):
        emoji = get_emoji_from_order(0)
        if getattr(emoji, "group", None) is not None:
            group = get_group(emoji)
            self.assertEqual(group.order, emoji.group)
            self.assertTrue(hasattr(group, "order"))

    def test_get_subgroup(self):
        emoji = get_emoji_from_order(0)
        if getattr(emoji, "subgroup", None) is not None:
            subgroup = get_subgroup(emoji)
            self.assertEqual(subgroup.order, emoji.subgroup)
            self.assertTrue(hasattr(subgroup, "order"))

    def test_get_all_groups(self):
        groups = get_all_groups()
        self.assertIsInstance(groups, list)
        self.assertGreaterEqual(len(groups), 1)
        self.assertEqual(groups[0].order, 0)
        for g in groups:
            self.assertTrue(hasattr(g, "order"))

    def test_get_all_subgroups(self):
        subgroups = get_all_subgroups()
        self.assertIsInstance(subgroups, list)
        self.assertGreaterEqual(len(subgroups), 1)
        self.assertEqual(subgroups[0].order, 0)
        for sg in subgroups:
            self.assertTrue(hasattr(sg, "order"))

    def test_emoji_kitchen(self):
        emoji1 = get_emoji_from_hexcode("1F600")
        emoji2 = get_emoji_from_hexcode("1F600")
        url = emoji_kitchen(emoji1, emoji2)
        self.assertTrue(url.startswith("https://"))

    def test_search_emojis_label(self):
        results = search_emojis("grinning")
        self.assertTrue(any(e.emoji == "😀" for e in results))
        results2 = search_emojis("GRINNING")
        self.assertTrue(any(e.emoji == "😀" for e in results2))

    def test_search_emojis_shortcode(self):
        results = search_emojis("grinning")
        self.assertTrue(any(e.emoji == "😀" for e in results))

    def test_search_emojis_hexcode(self):
        results = search_emojis("1f600")
        self.assertTrue(any(e.emoji == "😀" for e in results))
        results2 = search_emojis("1F600")
        self.assertTrue(any(e.emoji == "😀" for e in results2))

    def test_search_emojis_emoji(self):
        results = search_emojis("😀")
        self.assertTrue(any(e.emoji == "😀" for e in results))

    def test_search_emojis_tags(self):
        results = search_emojis("smile")
        # Accept True or False, depending on your emoji data
        self.assertIsInstance(results, list)

    def test_search_emojis_no_results(self):
        results = search_emojis("notarealemoji")
        self.assertEqual(results, [])
        results2 = search_emojis("")
        self.assertIsInstance(results2, list)

    def test_emojize_and_demojize_roundtrip(self):
        text = "Say :grinning: and :grinning:!"
        emojized = emojize(text)
        roundtrip = demojize(emojized)
        self.assertIn(":grinning:", roundtrip)
        text2 = "Say :unknown:!"
        emojized2 = emojize(text2)
        roundtrip2 = demojize(emojized2)
        self.assertIn(":unknown:", roundtrip2)

    def test_emojize_with_multiple_shortcodes(self):
        text = ":grinning: :grinning:"
        result = emojize(text)
        self.assertIn("😀", result)

    def test_demojize_with_skin_tone(self):
        text = "😁🏻"
        result = demojize(text)
        # Accept either the emoji or a shortcode
        self.assertTrue(result.startswith(":") or result == "😁🏻")

    def test_get_emoji_from_name_case(self):
        with self.assertRaises(ValueError):
            get_emoji_from_name("Grinning Face")

    def test_get_emoji_from_hexcode_case(self):
        with self.assertRaises(ValueError):
            get_emoji_from_hexcode("1f600")

    def test_get_emoji_from_shortcode_case(self):
        with self.assertRaises(ValueError):
            get_emoji_from_shortcode("Grinning")

    def test_get_emoji_from_order_out_of_bounds(self):
        with self.assertRaises(IndexError):
            get_emoji_from_order(-100)
        with self.assertRaises(IndexError):
            get_emoji_from_order(999999)

    def test_search_emojis_partial_label(self):
        results = search_emojis("grin")
        self.assertTrue(any("grin" in e.label for e in results))

    def test_search_emojis_partial_tag(self):
        results = search_emojis("smi")
        self.assertIsInstance(results, list)

    def test_search_emojis_partial_shortcode(self):
        results = search_emojis("grin")
        self.assertTrue(any("grin" in sc for e in results for sc in getattr(e, "shortcodes", [])))

    def test_emojize_edge_cases(self):
        self.assertEqual(emojize("::"), "::")
        self.assertEqual(emojize(": :"), ": :")
        self.assertEqual(emojize("::grinning::"), ":😀:")
        self.assertEqual(emojize("::::"), "::::")
        self.assertEqual(emojize("::grinning::grinning::"), ":😀😀:")
        self.assertEqual(emojize("::grinning: ::grinning:"), ":😀 :😀")

    def test_demojize_edge_cases(self):
        self.assertEqual(demojize("::"), "::")
        self.assertEqual(demojize("😀::"), ":grinning:::")
        self.assertEqual(demojize("😀😀"), ":grinning::grinning:")
        self.assertEqual(demojize("😀 😀"), ":grinning: :grinning:")
        self.assertEqual(demojize("::::"), "::::")
        self.assertEqual(demojize("😀::::"), ":grinning:::::")
        self.assertEqual(demojize("😀😀😀😀"), ":grinning::grinning::grinning::grinning:")

    def test_emojize_invalid_shortcodes(self):
        self.assertEqual(emojize(":notarealshortcode:"), ":notarealshortcode:")
        self.assertEqual(emojize(":grinning::notarealshortcode:"), "😀:notarealshortcode:")
        self.assertEqual(emojize(":notarealshortcode::grinning:"), ":notarealshortcode:😀")

    def test_demojize_unknown_emoji(self):
        # Accept either the emoji or the shortcode, depending on mapping
        ufo = demojize("🛸")
        self.assertTrue(ufo == "🛸" or ufo.startswith(":"))
        unicorn = demojize("🦄")
        self.assertTrue(unicorn == "🦄" or unicorn.startswith(":"))

    def test_emojize_and_demojize_empty(self):
        self.assertEqual(emojize(""), "")
        self.assertEqual(demojize(""), "")

    def test_emojize_and_demojize_only_punctuation(self):
        self.assertEqual(emojize("!!!"), "!!!")
        self.assertEqual(demojize("!!!"), "!!!")
        self.assertEqual(emojize("..."), "...")
        self.assertEqual(demojize("..."), "...")

    def test_emojize_and_demojize_whitespace(self):
        self.assertEqual(emojize(" "), " ")
        self.assertEqual(demojize(" "), " ")
        self.assertEqual(emojize("\t\n"), "\t\n")
        self.assertEqual(demojize("\t\n"), "\t\n")

if __name__ == "__main__":
    unittest.main()


"""
Emojis for Python
~~~~~~~~~~~~~~~~~~~

A basic emoji manipulator for Python.

:copyright: (c) 2025 Omkaar
:license: MIT, see LICENSE for more details.
"""


__title__ = "emojis.py"
__author__ = "Ombucha"
__license__ = "MIT"
__copyright__ = "Copyright 2025 Omkaar"
__version__ = "1.3.0"


from .emojis import *
from .utils import *



"""
MIT License

Copyright (c) 2025 Omkaar

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


import json
import os


with open(os.path.join(os.path.dirname(__file__), "database/data.json"), encoding="utf-8") as file:
    data = json.loads(file.read())

data_from_emoji = {}
for item in data:
    data_from_emoji[item["emoji"]] = item

    if "skins" in item:
        for skin in item["skins"]:
            data_from_emoji[skin["emoji"]] = skin

with open(os.path.join(os.path.dirname(__file__), "database/messages.json"), encoding="utf-8") as file:
    messages = json.loads(file.read())

group_from_order = {}
for item in messages["groups"]:
    group_from_order[item["order"]] = item

subgroup_from_order = {}
for item in messages["subgroups"]:
    subgroup_from_order[item["order"]] = item

with open(os.path.join(os.path.dirname(__file__), "database/shortcodes.json"), encoding="utf-8") as file:
    shortcodes = json.loads(file.read())


class Emoji:

    """
    A class that represents an emoji.

    Parameters
    ----------
    :param emoji: The emoji's symbol (character).
    :type emoji: str

    Attributes
    ----------
    :ivar emoji: The emoji character.
    :ivar label: The label or description of the emoji.
    :ivar hexcode: The Unicode hexcode of the emoji.
    :ivar gender: The gender associated with the emoji, if any.
    :ivar version: The Unicode version in which the emoji was introduced.
    :ivar order: The ordering index of the emoji.
    :ivar group: The group/category of the emoji.
    :ivar subgroup: The subgroup/category of the emoji.
    :ivar tags: A list of tags associated with the emoji.
    :ivar type: The type of emoji (e.g., "standard", "modifier").
    :ivar skins: A list of Emoji objects representing skin tone variations.
    :ivar shortcodes: A list of shortcodes for the emoji.
    """

    def __init__(self, emoji: str) -> None:

        try:
            _data = data_from_emoji[emoji]
        except KeyError as exc:
            raise ValueError("Emoji not found in the database.") from exc

        self.emoji = emoji
        self.label = _data.get("label", None)
        self.hexcode = _data.get("hexcode", None)
        self.gender = _data.get("gender", None)
        self.version = _data.get("version", None)
        self.order = _data.get("order", None)
        self.group = _data.get("group", None)
        self.subgroup = _data.get("subgroup", None)
        self.tags = _data.get("tags", [])
        self.type = _data.get("type", None)
        self.skins = [Emoji(skin["emoji"]) for skin in _data.get("skins", [])]

        self.shortcodes = shortcodes.get(self.hexcode, [])
        if isinstance(self.shortcodes, str):
            self.shortcodes = [self.shortcodes]

    def __str__(self) -> str:
        return self.emoji

    def __repr__(self) -> str:
        return f"Emoji(emoji={self.emoji!r}, label={self.label!r}, hexcode={self.hexcode!r})"

    def __int__(self) -> int:
        return self.order

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, Emoji):
            return NotImplemented
        return self.emoji == other.emoji

    def __hash__(self) -> int:
        return hash(self.emoji)

    def __lt__(self, other: object) -> bool:
        if not isinstance(other, Emoji):
            return NotImplemented
        return self.order < other.order

    def __le__(self, other: object) -> bool:
        if not isinstance(other, Emoji):
            return NotImplemented
        return self.order <= other.order

    def __gt__(self, other: object) -> bool:
        if not isinstance(other, Emoji):
            return NotImplemented
        return self.order > other.order

    def __ge__(self, other: object) -> bool:
        if not isinstance(other, Emoji):
            return NotImplemented
        return self.order >= other.order

    def __ne__(self, other: object) -> bool:
        if not isinstance(other, Emoji):
            return NotImplemented
        return self.emoji != other.emoji

class Group:

    """
    A class that represents a group of emojis.

    Parameters
    ----------
    :param order: The order index of the emoji group.
    :type order: int

    Attributes
    ----------
    :ivar order: The order index of the emoji group.
    :ivar name: The name of the emoji group.
    :ivar key: The key identifier for the group.
    :ivar emojis: A list of Emoji objects in this group.
    """

    def __init__(self, order: int) -> None:
        self.order = order
        self.name = group_from_order[order]["message"]
        self.key = group_from_order[order]["key"]
        self.emojis = {Emoji(key) for key, value in data_from_emoji.items() if value.get("group", None) == order}

    def __str__(self) -> str:
        return self.key

    def __repr__(self) -> str:
        return f"Group(name={self.name!r}, order={self.order!r})"

    def __int__(self) -> int:
        return self.order

    def __len__(self) -> int:
        return len(self.emojis)

    def __contains__(self, item: Emoji) -> bool:
        if not isinstance(item, Emoji):
            return NotImplemented
        return item in self.emojis

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, Group):
            return NotImplemented
        return self.order == other.order

    def __lt__(self, other: object) -> bool:
        if not isinstance(other, Group):
            return NotImplemented
        return self.order < other.order

    def __le__(self, other: object) -> bool:
        if not isinstance(other, Group):
            return NotImplemented
        return self.order <= other.order

    def __gt__(self, other: object) -> bool:
        if not isinstance(other, Group):
            return NotImplemented
        return self.order > other.order

    def __ge__(self, other: object) -> bool:
        if not isinstance(other, Group):
            return NotImplemented
        return self.order >= other.order

    def __ne__(self, other: object) -> bool:
        if not isinstance(other, Group):
            return NotImplemented
        return self.order != other.order

class Subgroup:

    """
    A class that represents a subgroup of emojis within a group.

    Parameters
    ----------
    :param order: The order index of the emoji subgroup.
    :type order: int

    Attributes
    ----------
    :ivar order: The order index of the emoji subgroup.
    :ivar name: The name of the emoji subgroup.
    :ivar key: The key identifier for the subgroup.
    :ivar emojis: A list of Emoji objects in this subgroup.
    """

    def __init__(self, order: int) -> None:
        self.order = order
        self.name = subgroup_from_order[order]["message"]
        self.key = subgroup_from_order[order]["key"]
        self.emojis = {Emoji(key) for key, value in data_from_emoji.items() if value.get("subgroup", None) == order}

    def __str__(self) -> str:
        return self.key

    def __repr__(self) -> str:
        return f"Subgroup(name={self.name!r}, order={self.order!r})"

    def __int__(self) -> int:
        return self.order

    def __len__(self) -> int:
        return len(self.emojis)

    def __contains__(self, item: Emoji) -> bool:
        if not isinstance(item, Emoji):
            return NotImplemented
        return item in self.emojis

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, Subgroup):
            return NotImplemented
        return self.order == other.order

    def __lt__(self, other: object) -> bool:
        if not isinstance(other, Subgroup):
            return NotImplemented
        return self.order < other.order

    def __le__(self, other: object) -> bool:
        if not isinstance(other, Subgroup):
            return NotImplemented
        return self.order <= other.order

    def __gt__(self, other: object) -> bool:
        if not isinstance(other, Subgroup):
            return NotImplemented
        return self.order > other.order

    def __ge__(self, other: object) -> bool:
        if not isinstance(other, Subgroup):
            return NotImplemented
        return self.order >= other.order

    def __ne__(self, other: object) -> bool:
        if not isinstance(other, Subgroup):
            return NotImplemented
        return self.order != other.order



"""
MIT License

Copyright (c) 2025 Omkaar

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


# pylint: skip-file

import unittest

from emojis.utils import (
    get_all_emojis,
    get_emoji_from_name,
    get_emoji_from_hexcode,
    get_emoji_from_shortcode,
    get_emoji_from_order,
    search_emojis,
)

class TestEmojis(unittest.TestCase):

    def test_get_all_emojis_returns_list(self):
        emojis = get_all_emojis()
        self.assertIsInstance(emojis, list)
        self.assertGreater(len(emojis), 0)
        for e in emojis:
            self.assertTrue(hasattr(e, "emoji"))
            self.assertTrue(isinstance(e.emoji, str))
            self.assertGreater(len(e.emoji), 0)

    def test_get_emoji_from_name_valid(self):
        emoji = get_emoji_from_name("grinning face")
        self.assertEqual(emoji.emoji, "😀")

    def test_get_emoji_from_name_invalid(self):
        with self.assertRaises(ValueError):
            get_emoji_from_name("not an emoji")
        with self.assertRaises(ValueError):
            get_emoji_from_name("")
        with self.assertRaises(ValueError):
            get_emoji_from_name(" ")
        with self.assertRaises(ValueError):
            get_emoji_from_name(":grinning:")
        # If you don't check type, just ensure it doesn't crash
        try:
            get_emoji_from_name(123)
        except Exception:
            pass
        try:
            get_emoji_from_name(None)
        except Exception:
            pass

    def test_get_emoji_from_name_case(self):
        with self.assertRaises(ValueError):
            get_emoji_from_name("Grinning Face")

    def test_get_emoji_from_hexcode_valid(self):
        emoji = get_emoji_from_hexcode("1F600")
        self.assertEqual(emoji.emoji, "😀")

    def test_get_emoji_from_hexcode_invalid(self):
        with self.assertRaises(ValueError):
            get_emoji_from_hexcode("FFFF")
        with self.assertRaises(ValueError):
            get_emoji_from_hexcode("")
        with self.assertRaises(ValueError):
            get_emoji_from_hexcode("not_a_hexcode")
        with self.assertRaises(ValueError):
            get_emoji_from_hexcode("1f600")  # lowercase
        try:
            get_emoji_from_hexcode(123)
        except Exception:
            pass
        try:
            get_emoji_from_hexcode(None)
        except Exception:
            pass

    def test_get_emoji_from_shortcode_valid(self):
        emoji = get_emoji_from_shortcode("grinning")
        self.assertEqual(emoji.emoji, "😀")

    def test_get_emoji_from_shortcode_invalid(self):
        with self.assertRaises(ValueError):
            get_emoji_from_shortcode("not_a_shortcode")
        with self.assertRaises(ValueError):
            get_emoji_from_shortcode("")
        with self.assertRaises(ValueError):
            get_emoji_from_shortcode(" ")
        with self.assertRaises(ValueError):
            get_emoji_from_shortcode("Grinning")  # case-sensitive
        try:
            get_emoji_from_shortcode(123)
        except Exception:
            pass
        try:
            get_emoji_from_shortcode(None)
        except Exception:
            pass

    def test_get_emoji_from_order_valid(self):
        emoji = get_emoji_from_order(0)
        self.assertTrue(isinstance(emoji.emoji, str))
        self.assertGreater(len(emoji.emoji), 0)
        try:
            emoji2 = get_emoji_from_order(1)
            self.assertTrue(isinstance(emoji2.emoji, str))
            self.assertGreater(len(emoji2.emoji), 0)
        except Exception:
            pass

    def test_get_emoji_from_order_invalid(self):
        with self.assertRaises(IndexError):
            get_emoji_from_order(-1)
        with self.assertRaises(IndexError):
            get_emoji_from_order(999999)
        with self.assertRaises(IndexError):
            get_emoji_from_order(-100)
        # If you don't check type, just ensure it doesn't crash
        try:
            get_emoji_from_order("zero")
        except Exception:
            pass
        try:
            get_emoji_from_order(None)
        except Exception:
            pass
        try:
            get_emoji_from_order(1.5)
        except Exception:
            pass

    def test_search_emojis_by_label(self):
        results = search_emojis("grinning")
        self.assertTrue(any(e.emoji == "😀" for e in results))

    def test_search_emojis_by_shortcode(self):
        results = search_emojis("grin")
        self.assertTrue(any("grin" in sc for e in results for sc in getattr(e, "shortcodes", [])))

    def test_search_emojis_by_hexcode(self):
        results = search_emojis("1F600")
        self.assertTrue(any(e.emoji == "😀" for e in results))
        results2 = search_emojis("1f600")
        self.assertTrue(any(e.emoji == "😀" for e in results2))

    def test_search_emojis_by_emoji(self):
        results = search_emojis("😀")
        self.assertTrue(any(e.emoji == "😀" for e in results))

    def test_search_emojis_no_results(self):
        results = search_emojis("notarealemoji")
        self.assertEqual(results, [])
        results2 = search_emojis("")
        self.assertIsInstance(results2, list)

    def test_search_emojis_partial_match(self):
        results = search_emojis("grin")
        self.assertTrue(any("grin" in e.label for e in results))

    def test_search_emojis_whitespace(self):
        results = search_emojis(" ")
        self.assertIsInstance(results, list)

    def test_search_emojis_special_characters(self):
        results = search_emojis("!@#$%^&*()")
        self.assertIsInstance(results, list)
        self.assertEqual(results, [])

    def test_search_emojis_type_errors(self):
        # If your implementation does not raise, just check for list or None
        try:
            result = search_emojis(None)
        except Exception:
            result = None
        self.assertTrue(result is None or isinstance(result, list))
        try:
            result = search_emojis(123)
        except Exception:
            result = None
        self.assertTrue(result is None or isinstance(result, list))

    def test_get_emoji_from_order_large(self):
        # Test very large positive and negative numbers
        with self.assertRaises(IndexError):
            get_emoji_from_order(10**10)
        with self.assertRaises(IndexError):
            get_emoji_from_order(-10**10)

    def test_search_emojis_numeric_string(self):
        # Should not raise, but may return empty
        results = search_emojis("123456")
        self.assertIsInstance(results, list)

    def test_search_emojis_unicode(self):
        # Unicode string that is not an emoji
        results = search_emojis("你好")
        self.assertIsInstance(results, list)

if __name__ == "__main__":
    unittest.main()



"""
MIT License

Copyright (c) 2025 Omkaar

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


import json
import os

from .emojis import Emoji, Group, Subgroup


with open(os.path.join(os.path.dirname(__file__), "database/data.json"), encoding="utf-8") as file:
    data = json.loads(file.read())

data_from_emoji = {}
for item in data:
    data_from_emoji[item["emoji"]] = item

    if "skins" in item:
        for skin in item["skins"]:
            data_from_emoji[skin["emoji"]] = skin

with open(os.path.join(os.path.dirname(__file__), "database/metadata.json"), encoding="utf-8") as file:
    metadata = json.loads(file.read())

with open(os.path.join(os.path.dirname(__file__), "database/shortcodes.json"), encoding="utf-8") as file:
    shortcodes = json.loads(file.read())

shortcodes_from_shortcode = {}
for item in shortcodes:
    if isinstance(shortcodes[item], str):
        shortcodes[item] = [shortcodes[item]]
    for shortcode in shortcodes[item]:
        shortcodes_from_shortcode[shortcode] = item


def is_emoji(character: str) -> bool:
    """
    Check if a character is an emoji.

    :param character: The character to check.
    :type character: str
    :return: True if the character is an emoji, False otherwise.
    :rtype: bool
    """
    return character in data_from_emoji

def emoji_count(text: str) -> int:
    """
    Count the number of emojis in a given text.

    :param text: The text to count emojis in.
    :type text: str
    :return: The number of emojis in the text.
    :rtype: int
    """
    return sum(1 for char in text if is_emoji(char))

def get_emoji_from_name(name: str) -> Emoji:
    """
    Get the emoji character from its name.

    :param name: The name of the emoji.
    :type name: str
    :return: An Emoji object if found.
    :rtype: Emoji
    """
    for item in data:
        if item["label"] == name:
            return Emoji(item["emoji"])
    raise ValueError(f"Emoji with name '{name}' not found.")

def get_emoji_from_hexcode(hexcode: str) -> Emoji:
    """
    Get the emoji character from its hexcode.

    :param hexcode: The hexcode of the emoji.
    :type hexcode: str
    :return: An Emoji object if found.
    :rtype: Emoji
    """
    for item in data:
        if item["hexcode"] == hexcode:
            return Emoji(item["emoji"])
    raise ValueError(f"Emoji with hexcode '{hexcode}' not found.")

def get_emoji_from_shortcode(shortcode: str) -> Emoji:
    """
    Get the emoji character from its shortcode.

    :param shortcode: The shortcode of the emoji.
    :type shortcode: str
    :return: An Emoji object if found.
    :rtype: Emoji
    """
    hexcode = shortcodes_from_shortcode.get(shortcode)
    if hexcode:
        return get_emoji_from_hexcode(hexcode)
    raise ValueError(f"Emoji with shortcode '{shortcode}' not found.")

def get_emoji_from_order(order: int) -> Emoji:
    """
    Get the emoji character from its order in the database.

    :param order: The order index of the emoji.
    :type order: int
    :return: An Emoji object if found.
    :rtype: Emoji
    """
    if 0 <= order < len(data):
        return Emoji(data[order]["emoji"])
    raise IndexError(f"Order '{order}' is out of range.")

def get_all_emojis() -> list:
    """
    Get a list of all emoji characters.

    :return: A list of all emoji characters.
    :rtype: list
    """
    return [Emoji(item["emoji"]) for item in data]

def emojize(text: str) -> str:
    """
    Convert shortcodes in the text to emojis.

    :param text: The text containing shortcodes.
    :type text: str
    :return: The text with shortcodes replaced by emojis.
    :rtype: str
    """
    for key, value in shortcodes_from_shortcode.items():
        text = text.replace(f":{key}:",  "".join(chr(int(emoji, 16)) for emoji in value.split("-")))
    return text

def demojize(text: str) -> str:
    """
    Convert emojis in the text to shortcodes.

    .. note::

            This function only replaces emojis with the alphabetically first shortcode if multiple shortcodes exist for the same emoji.

    :param text: The text containing emojis.
    :type text: str
    :return: The text with emojis replaced by shortcodes.
    :rtype: str
    """
    for key, value in shortcodes.items():
        text = text.replace("".join(chr(int(emoji, 16)) for emoji in key.split("-")), f":{value[0]}:")
    return text

def get_group(emoji: Emoji) -> Group:
    """
    Get the group of an emoji.

    :param emoji: The Emoji object.
    :type emoji: Emoji
    :return: The Group object containing the emoji.
    :rtype: Group
    """
    return Group(emoji.group)

def get_subgroup(emoji: Emoji) -> Subgroup:
    """
    Get the subgroup of an emoji.

    :param emoji: The Emoji object.
    :type emoji: Emoji
    :return: The Subgroup object containing the emoji.
    :rtype: Subgroup
    """
    return Subgroup(emoji.subgroup)

def get_all_groups() -> list:
    """
    Get a list of all emoji groups.

    :return: A list of Group objects.
    :rtype: list
    """
    return [Group(order) for order in range(10)]

def get_all_subgroups() -> list:
    """
    Get a list of all emoji subgroups.

    :return: A list of Subgroup objects.
    :rtype: list
    """
    return [Subgroup(order) for order in range(100)]

def emoji_kitchen(emoji1: Emoji, emoji2: Emoji) -> str:
    """
    Combine two emojis to create a new emoji.

    .. note::

            This function utilises Google's Emoji Kitchen feature to combine two emojis.

    :param emoji1: The first Emoji object.
    :type emoji1: Emoji
    :param emoji2: The second Emoji object.
    :type emoji2: Emoji
    :return: The link to the combined emoji.
    :rtype: str
    """
    return metadata["data"][emoji1.hexcode.lower()]["combinations"][emoji2.hexcode.lower()][0]["gStaticUrl"]

def search_emojis(query: str) -> list:
    """
    Search for emojis by name or shortcode.

    :param query: The search query (name or shortcode).
    :type query: str
    :return: A list of Emoji objects that match the query.
    :rtype: list
    """
    results = []
    for item in get_all_emojis():
        if query.lower() in item.label.lower() or query.lower() in item.shortcodes or query.lower() in item.hexcode.lower() or query.lower() in item.emoji or query.lower() in item.tags:
            results.append(item)
    return results



from pathlib import Path

from setuptools import setup

HERE = Path(__file__).resolve().parent
README = (HERE / "README.rst").read_text()

setup(
    name = "emojis.py",
    version = "1.3.0",
    description = "A basic emoji manipulator for Python.",
    package_data = {"emojis.py": ["emojis/database/*.json"]},
    long_description = README,
    long_description_content_type = "text/x-rst",
    url = "https://github.com/Ombucha/emojis.py",
    author = "Omkaar",
    author_email = "omkaar.nerurkar@gmail.com",
    license = "MIT",
    classifiers = [
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
    ],
    python_requires='>= 3.8.0',
    packages = ["emojis"],
    include_package_data = True
)
